const cron = require('node-cron');
const db = require("../infra/database/db");
const { sendMessage, sendImage } = require("./gzappy");  

if (process.env.NODE_ENV === "production") {
    async function lembreteDePedido() {
        console.log('Executando a tarefa diária');
        try {
            // Função que busca usuários que NÃO fizeram pedidos hoje
            const pedidosNaoFeitos = await new Promise((resolve, reject) => {
                db.query(
                    `SELECT u.*
                    FROM user u
                    LEFT JOIN pedidos p ON u.id = p.id_user AND DATE(p.data) = CURDATE()
                    WHERE p.id_user IS NULL 
                    AND u.status = 'aprovado' 
                    AND u.role != 'ADMIN' 
                    AND u.dias_pedido LIKE CONCAT('%', 
                        CASE DAYOFWEEK(NOW())
                            WHEN 1 THEN 'Domingo'
                            WHEN 2 THEN 'Segunda'
                            WHEN 3 THEN 'Terça'
                            WHEN 4 THEN 'Quarta'
                            WHEN 5 THEN 'Quinta'
                            WHEN 6 THEN 'Sexta'
                            WHEN 7 THEN 'Sábado'
                        END, 
                    '%')`, [],
                    (error, results) => {
                        if (error) return reject(error);
                        resolve(results);
                    }
                );
            });

            // Função que busca mensagens vinculadas a usuários com horário agendado
            const lembretes = await new Promise((resolve, reject) => {
                db.query(
                    `SELECT mv.id_user, mv.id_mensagem, mv.horario, u.celular, u.email, u.notificacao_email, u.notificacao_whats
                    FROM mensagens_vinculadas mv
                    JOIN user u ON mv.id_user = u.id
                    WHERE u.status = 'aprovado' AND u.role != 'ADMIN'`,
                    [],
                    (error, results) => {
                        if (error) return reject(error);
                        resolve(results);
                    }
                );
            });

            // Função que busca mensagens 
            async function buscarMensagem(id_mensagem) {
                return new Promise((resolve, reject) => {
                    db.query(
                        `SELECT mensagem, image FROM mensagens WHERE id = ?`,
                        [id_mensagem],
                        (error, results) => {
                            if (error) return reject(error);
                            resolve(results[0]); // ← retorna o objeto inteiro
                        }
                    );
                });
            }
            

            // Função quem ja fez pedido 
            async function fezPedidoHoje(id_user) {
                return new Promise((resolve, reject) => {
                    db.query(
                        `SELECT 1 FROM pedidos WHERE id_user = ? AND DATE(data) = CURDATE()`,
                        [id_user],
                        (error, results) => {
                            if (error) return reject(error);
                            resolve(results.length > 0);
                        }
                    );
                });
            }

            // Função que envia o lembrete
            async function enviarLembrete({ id_user, id_mensagem, celular }) {
                try {
                    const dadosMensagem = await buscarMensagem(id_mensagem);
                    console.log('dadosMensagem:', dadosMensagem)
                    if (!dadosMensagem?.mensagem) {
                        console.log(`Mensagem não encontrada para id ${id_mensagem}`);
                        return;
                    }
            
                    const numeroFormatado = `55${celular}`;
            
                    if (dadosMensagem.image) {
                        await sendImage(numeroFormatado, dadosMensagem.mensagem, dadosMensagem.image);
                        console.log(`Lembrete com imagem enviado para usuário ${id_user}`);
                    } else {
                        await sendMessage(dadosMensagem.mensagem, numeroFormatado);
                        console.log(`Lembrete de texto enviado para usuário ${id_user}`);
                    }
                } catch (error) {
                    console.error(`Erro ao enviar lembrete para ${id_user}:`, error);
                }
            }

            // Enviar lembretes para usuários que não fizeram pedidos
            for (const user of pedidosNaoFeitos) {
                const numeroFormatado = `55${user.celular}`;
                const mensagemPadrao = `Bom dia, tudo bem?,\n
passando para lembrar que vamos ter entregas amanhã na sua região.\n
NÃO ESQUEÇA DE FAZER O PEDIDO ATRÁVES DO LINK:\n
https://aliancapedidos.com.br`;

                // Se não tiver lembrete vinculado, envia às 15:15
                const usuarioComLembrete = lembretes.some(lembrete => lembrete.id_user === user.id);

                if (!usuarioComLembrete) {
                    if (user.notificacao_whats !== 1) {
                        console.log(`WhatsApp desativado para usuário ${user.id}. Não será agendado às 15:15.`);
                        return;
                    }
                    console.log('Usuário que não fez pedido: ', user.razao_social)
                    cron.schedule('15 15 * * *', async () => {
                        const jaFezPedido = await fezPedidoHoje(user.id);
                    
                        if (jaFezPedido) {
                            console.log(`Usuário ${user.id} fez pedido hoje. Nada será enviado às 15:15.`);
                            return;
                        }

                        await sendMessage(mensagemPadrao, numeroFormatado);
                        console.log(`Lembrete enviado para ${user.celular} às 15:15`);
                    }, {
                        scheduled: true,
                        timezone: "America/Sao_Paulo"
                    });
                }
            }

            // Função que agendará o lembrete usando cron
            lembretes.forEach(async ({ id_user, id_mensagem, horario, celular, notificacao_whats }) => {
                if (notificacao_whats !== 1) {
                    console.log(`WhatsApp desativado para usuário ${id_user}. Não será agendado às ${horario}.`);
                    return;
                }                
            
                const user = pedidosNaoFeitos.find(u => u.id === id_user);
            
                if (!user) {
                    // Buscar dias_pedido desse usuário
                    const usuario = await new Promise((resolve, reject) => {
                        db.query(
                            `SELECT dias_pedido FROM user WHERE id = ?`,
                            [id_user],
                            (error, results) => {
                                if (error) return reject(error);
                                resolve(results[0]);
                            }
                        );
                    });
            
                    const diasPedido = usuario?.dias_pedido || '';
                    const diaAtual = new Date().toLocaleDateString('pt-BR', { weekday: 'long' });
                    const podeReceberHoje = diasPedido.includes(diaAtual);
            
                    const jaFezPedido = await fezPedidoHoje(id_user);
                    if (!podeReceberHoje || jaFezPedido) return;
                }
            
                const [hora, minuto] = horario.split(':');
            
                cron.schedule(`${minuto} ${hora} * * *`, () => {
                    enviarLembrete({ id_user, id_mensagem, celular });
                });
            
                console.log(`Lembrete agendado para ${celular} às ${horario}`);
            });
            

        } catch (error) {
            console.error("Erro ao processar lembretes:", error);
        }
    }

    async function buscarMensagensPromocionais() {
        return new Promise((resolve, reject) => {
            db.query(`
                SELECT id, mensagem, data, hora
                FROM mensagens
                WHERE padrao = false AND data IS NOT NULL AND hora IS NOT NULL
            `, [], (err, results) => {
                if (err) return reject(err);
                resolve(results);
            });
        });
    }
    
    async function buscarUsuariosAprovados() {
        return new Promise((resolve, reject) => {
            db.query(`
                SELECT id, celular, email, notificacao_email
                FROM user
                WHERE status = 'aprovado' AND role != 'ADMIN'
            `, [], (err, results) => {
                if (err) return reject(err);
                resolve(results);
            });
        });
    }
    
    // Sempre que o servidor reiniciar essa função será chamada
    // e agendará todas as mensagens promocionais que estão na tabela mensagens
    async function agendarMensagensPromocionaisParaTodos() {
        try {
            const mensagens = await buscarMensagensPromocionais();
            const usuarios = await buscarUsuariosAprovados();
            
            for (const mensagem of mensagens) {
                const [hora, minuto] = mensagem.hora.split(":");
                const data = new Date(mensagem.data);
                const dia = data.getDate();
                const mes = data.getMonth() + 1; // Janeiro = 0

    
                const cronPattern = `${minuto} ${hora} ${parseInt(dia)} ${parseInt(mes)} *`;
    
                cron.schedule(cronPattern, async () => {
                    for (const user of usuarios) {
                        const numeroFormatado = `55${user.celular}`;
                        await sendMessage(mensagem.mensagem, numeroFormatado);
                        console.log(`Mensagem promocional enviada para ${user.celular}`);
    
                        // if (user.notificacao_email) {
                        //     await enviarEmail(user.email, "Promoção", "../view/promocional.html");
                        // }
                    }
                }, {
                    scheduled: true,
                    timezone: "America/Sao_Paulo"
                });
    
                // console.log(`✅ Mensagem agendada para ${mensagem.data} às ${mensagem.hora}`);
            }
    
        } catch (error) {
            console.error("Erro ao agendar mensagens promocionais:", error);
        }
    }

    // Sempre que a função de criar uma mensagem for chamada
    // essa função será chamada e agendará a mensagem promocional
    // evitando perca de mensagens promocionais criadas depois do servidor reiniciar
    function agendarMensagemPromocional(mensagem) {
        const [ano, mes, dia] = mensagem.data.split("-");
        const [hora, minuto] = mensagem.hora.split(":");
        const cronPattern = `${minuto} ${hora} ${parseInt(dia)} ${parseInt(mes)} *`;
    
        db.query(
            `SELECT celular, email, notificacao_email, notificacao_whats 
             FROM user 
             WHERE status = 'aprovado' AND role != 'ADMIN' AND notificacao_whats = 1`,
            (err, usuarios) => {
                if (err) return console.error("Erro ao buscar usuários:", err);

                console.log('Usuarios: ', usuarios)
    
                cron.schedule(cronPattern, async () => {
                    for (const user of usuarios) {
                        const numeroFormatado = `55${user.celular}`;
                
                        try {
                            if (mensagem.image) {
                                await sendImage(numeroFormatado, mensagem.mensagem, mensagem.image);
                                console.log(`📷 Mensagem com imagem enviada para ${user.celular}`);
                            } else {
                                await sendMessage(mensagem.mensagem, numeroFormatado);
                                console.log(`💬 Mensagem enviada para ${user.celular}`);
                            }
                        } catch (err) {
                            console.error(`❌ Erro ao enviar para ${user.celular}:`, err);
                        }
                    }
                }, {
                    scheduled: true,
                    timezone: "America/Sao_Paulo"
                });
                
    
                console.log(`📅 Mensagem promocional agendada para ${mensagem.data} às ${mensagem.hora}`);
            }
        );
    }

    cron.schedule('30 07 * * *', lembreteDePedido, {
        scheduled: true,
        timezone: "America/Sao_Paulo"
    });

  module.exports = { lembreteDePedido, agendarMensagensPromocionaisParaTodos, agendarMensagemPromocional };
} else {
    console.log("O ambiente não é de produção. Tarefas cron não serão agendadas") 
}
